//=============================================================================
// UPak_UPakRocket
//=============================================================================
class UPak_UPakRocket expands Projectile;

var() bool bAlwaysSynchronizeClient;

var float InitialSpeed;
var float RocketTrailTime;

var bool bPostBeginPlayExecuted; // for old buggy versions that call PostBeginPlay twice

// replicated built-in vectors hold inexact values, float values are replicated precisely
struct ReplicatedVector
{
	var float X, Y, Z;
};

var bool bSynchronizeClient;
var ReplicatedVector ReplicatedLocation;
var ReplicatedVector ReplicatedVelocity;
var ReplicatedVector ReplicatedAcceleration;

var UGoldFixSharedInfo SharedInfo;

replication
{
	reliable if (Role == ROLE_Authority)
		SharedInfo;

	reliable if (Role == ROLE_Authority)
		InitialSpeed;

	reliable if (Role == ROLE_Authority)
		ReplicatedLocation,
		ReplicatedVelocity,
		ReplicatedAcceleration;
}

simulated function PostBeginPlay()
{
	if (bPostBeginPlayExecuted)
		return;
	bPostBeginPlayExecuted = true;

	if (Level.NetMode == NM_Client)
	{
		if (bAlwaysSynchronizeClient)
			DrawType = DT_None;
		return;
	}

	class'UGoldFixSharedInfoContainer'.static.Propagate(UGoldFixSharedInfoContainer(Owner), self, SharedInfo);

	if (SharedInfo == none || SharedInfo.LoadedResources == none)
	{
		Destroy();
		return;
	}

	Mesh = SharedInfo.LoadedResources.class_UPakRocket.default.Mesh;

	if (Level.Game.IsA('DeathMatchGame'))
		Damage -= 35;
}


// ================================================================================================
// Flying State
// ================================================================================================

auto state Flying
{
	function Explode( vector HitLocation, vector HitNormal )
	{
		local UPak_UPakRocket_Explosion explosion_info;

		ControlledHurtRadius( Damage, 340.0, 'exploded', MomentumTransfer, HitLocation );

		explosion_info = Spawn(class'UPak_UPakRocket_Explosion');
		explosion_info.AssignSourceLocation(Location);
		explosion_info.AssignHitLocation(HitLocation);
		explosion_info.AssignHitNormal(HitNormal);
		explosion_info.SharedInfo = SharedInfo;

 		Destroy();
	}

	function ProcessTouch( Actor Other, Vector HitLocation )
	{
		if (Other != instigator && Rocket(Other) == none) 
		{
			if (Pawn(Other) != none && Pawn(Other) == Instigator)
			{
				Damage *= 0.25;
			}
			Explode(HitLocation, Normal(HitLocation - Other.Location));
		}
	}


	function BeginState()
	{
		Velocity = speed * vector(Rotation);
		InitialSpeed = speed;
		Acceleration = vector(Rotation) * 50;
		PlayAnim('Popup', 0.001);

		if (Region.Zone.bWaterZone)
		{
			Velocity = 0.6 * Velocity;
			AmbientSound = SharedInfo.LoadedResources.sound_RocketLoop2;
		}
		else
			AmbientSound = SharedInfo.LoadedResources.sound_RocketLoop1;

		PlaySound(SharedInfo.LoadedResources.sound_RocketShot1, SLOT_None);

		if (bAlwaysSynchronizeClient)
			Server_UpdateMovement();
	}

	simulated function ZoneChange(Zoneinfo NewZone)
	{
		local WaterRing w;

		if (VSize(NewZone.ZoneVelocity) != 0)
		{
			if (Level.NetMode == NM_Client)
				return;
			bSynchronizeClient = true;
		}

		if (Region.Zone.bWaterZone != NewZone.bWaterZone)
		{
			if (Level.NetMode != NM_Client)
			{
				w = Spawn(class'WaterRing',,,,rot(16384,0,0));
				w.DrawScale = 0.2;
			}

			if (NewZone.bWaterZone)
			{
				Velocity = 0.6 * Velocity;
				if (Level.NetMode != NM_Client)
					AmbientSound = SharedInfo.LoadedResources.sound_RocketLoop2;
			}
			else if (Level.NetMode != NM_Client)
			{
				PlayAnim('Wings', 0.15);
				AmbientSound = SharedInfo.LoadedResources.sound_RocketLoop1;
			}
		}
		if (bSynchronizeClient)
			Server_UpdateMovement();
	}
}


//
// Hurt actors within the radius.
//
function ControlledHurtRadius( float DamageAmount, float DamageRadius, name DamageName, float Momentum, vector HitLocation )
{
	local actor Victims;
	local float damageScale, dist;
	local vector dir;
	
	if( bHurtEntry )
		return;

	bHurtEntry = true;
	foreach VisibleCollidingActors( class 'Actor', Victims, DamageRadius, HitLocation )
	{
		if( Victims != self )
		{
			dir = Victims.Location - HitLocation;
			dist = FMax(1,VSize(dir));
			dir = dir/dist; 
			damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);
			if( Pawn( Victims ) != none && Pawn( Victims ) == Instigator )
			{
				DamageScale *= 0.5;
				Momentum *= 5;
			}
			Victims.TakeDamage(	damageScale * DamageAmount, Instigator,  Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir, ( ( damageScale * 4 )* ( Momentum * 0.75 ) * dir), DamageName	);
		} 
	}
	bHurtEntry = false;
}



simulated function Tick(float DeltaTime)
{
	if (Level.NetMode != NM_Client)
	{
		if (Region.Zone.bWaterZone && AnimSequence != 'Wings')
			LoopAnim('Wings');
		if (!Region.Zone.bWaterZone && VSize(Velocity) > 1200 && AnimSequence != 'Fly')
			LoopAnim('Fly');
	}
	if (!Region.Zone.bWaterZone)
	{
		Velocity *= 1.5;
		Acceleration = vector(Rotation) * 50;
	}
	else if (VSize(Velocity) < InitialSpeed)
		Velocity = Normal(Velocity) * InitialSpeed;

	if (bSynchronizeClient)
		Server_UpdateMovement();
	bSynchronizeClient = false;

	if (Level.NetMode == NM_DedicatedServer)
		return;

	Client_UpdateMovement();
	MakeRocketTrail(DeltaTime);
}

simulated function MakeRocketTrail(float DeltaTime)
{
	local SpriteSmokePuff smoke_puff;
	local Actor bubble_trail;
	local float RocketTrailInterval;

	RocketTrailInterval = 0.015;

	RocketTrailTime += DeltaTime;
	if (RocketTrailTime < RocketTrailInterval)
		return;

	RocketTrailTime -= RocketTrailInterval;
	if (RocketTrailTime > RocketTrailInterval)
		RocketTrailTime = RocketTrailInterval;

	if (Region.Zone.bWaterZone)
	{
		if (SharedInfo != none && SharedInfo.LoadedResources != none)
			bubble_trail = Spawn(SharedInfo.LoadedResources.class_BubbleTrail,,, Location);
		if (bubble_trail != none)
		{
			bubble_trail.RemoteRole = ROLE_None;
			bubble_trail.ScaleGlow = 0.5;
		}
	}
	else
	{
		smoke_puff = Spawn(class'RisingSpriteSmokePuff',,, Location);
		if (smoke_puff != none)
		{
			smoke_puff.RemoteRole = ROLE_None;
			smoke_puff.LifeSpan = 1.5 + Rand( 8.5 );
			smoke_puff.DrawScale += FRand();
		}
	}
}

function Server_UpdateMovement()
{
	if (Level.NetMode != NM_Standalone)
	{
		ReplicatedLocation = ConvertToReplicatedVector(Location);
		ReplicatedVelocity = ConvertToReplicatedVector(Velocity);
		ReplicatedAcceleration = ConvertToReplicatedVector(Acceleration);
	}
}

simulated function Client_UpdateMovement()
{
	if (Level.NetMode == NM_Client)
	{
		if (VSize(ConvertReplicatedVector(ReplicatedLocation)) != 0)
		{
			DrawType = DT_Mesh;
			SetLocation(ConvertReplicatedVector(ReplicatedLocation));
			ReplicatedLocation = ConvertToReplicatedVector(vect(0, 0, 0));
		}
		if (VSize(ConvertReplicatedVector(ReplicatedVelocity)) != 0)
		{
			Velocity = ConvertReplicatedVector(ReplicatedVelocity);
			Acceleration = ConvertReplicatedVector(ReplicatedAcceleration);
			if (!bFixedRotationDir)
				SetRotation(rotator(Velocity));
			ReplicatedVelocity = ConvertToReplicatedVector(vect(0, 0, 0));
		}
	}
}

static function ReplicatedVector ConvertToReplicatedVector(vector pos)
{
	local ReplicatedVector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

static function vector ConvertReplicatedVector(ReplicatedVector pos)
{
	local vector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

defaultproperties
{
				speed=850.000000
				MaxSpeed=2500.000000
				Damage=80.000000
				MomentumTransfer=8500
				bNetTemporary=False
				RemoteRole=ROLE_SimulatedProxy
				LifeSpan=0.000000
				AnimSequence="Armed"
				AmbientGlow=96
				bUnlit=True
				bAlwaysRelevant=True
				bGameRelevant=False
				SoundRadius=128
				SoundVolume=255
				CollisionRadius=8.500000
				CollisionHeight=2.000000
				bProjTarget=True
				bBounce=True
				bFixedRotationDir=True
				RotationRate=(Roll=95000)
}
